@@ -8,19 +8,20 @@
  *
  * Copyright 2007 Rainer Gerhards and Adiscon GmbH.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This file is part of rsyslog.
  *
- * This program is distributed in the hope that it will be useful,
+ * Rsyslog is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * Rsyslog is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * along with Rsyslog.  If not, see <http://www.gnu.org/licenses/>.
  *
  * A copy of the GPL can be found in the file "COPYING" in this distribution.
  */
@@ -42,10 +43,14 @@
 #include "template.h"
 #include "ommysql.h"
 #include "module-template.h"
+#include "errmsg.h"
+
+MODULE_TYPE_OUTPUT
 
 /* internal structures
  */
 DEF_OMOD_STATIC_DATA
+DEFobjCurrIf(errmsg)
 
 typedef struct _instanceData {
 	MYSQL	*f_hmysql;		/* handle to MySQL */
@@ -75,7 +80,7 @@ ENDisCompatibleWithFeature
  */
 static void closeMySQL(instanceData *pData)
 {
-	assert(pData != NULL);
+	ASSERT(pData != NULL);
 
 	if(pData->f_hmysql != NULL) {	/* just to be on the safe side... */
 		mysql_server_end();
@@ -90,27 +95,12 @@ CODESTARTfreeInstance
 ENDfreeInstance
 
 
-BEGINneedUDPSocket
-CODESTARTneedUDPSocket
-ENDneedUDPSocket
-
-
 BEGINdbgPrintInstInfo
 CODESTARTdbgPrintInstInfo
 	/* nothing special here */
 ENDdbgPrintInstInfo
 
 
-BEGINonSelectReadyWrite
-CODESTARTonSelectReadyWrite
-ENDonSelectReadyWrite
-
-
-BEGINgetWriteFDForSelect
-CODESTARTgetWriteFDForSelect
-ENDgetWriteFDForSelect
-
-
 /* log a database error with descriptive message.
  * We check if we have a valid MySQL handle. If not, we simply
  * report an error, but can not be specific. RGerhards, 2007-01-30
@@ -120,12 +110,12 @@ static void reportDBError(instanceData *pData, int bSilent)
 	char errMsg[512];
 	unsigned uMySQLErrno;
 
-	assert(pData != NULL);
+	ASSERT(pData != NULL);
 
 	/* output log message */
 	errno = 0;
 	if(pData->f_hmysql == NULL) {
-		logerror("unknown DB error occured - could not obtain MySQL handle");
+		errmsg.LogError(NO_ERRCODE, "unknown DB error occured - could not obtain MySQL handle");
 	} else { /* we can ask mysql for the error description... */
 		uMySQLErrno = mysql_errno(pData->f_hmysql);
 		snprintf(errMsg, sizeof(errMsg)/sizeof(char), "db error (%d): %s\n", uMySQLErrno,
@@ -134,7 +124,7 @@ static void reportDBError(instanceData *pData, int bSilent)
 			dbgprintf("mysql, DBError(silent): %s\n", errMsg);
 		else {
 			pData->uLastMySQLErrno = uMySQLErrno;
-			logerror(errMsg);
+			errmsg.LogError(NO_ERRCODE, "%s", errMsg);
 		}
 	}
 		
@@ -150,12 +140,12 @@ static rsRetVal initMySQL(instanceData *pData, int bSilent)
 {
 	DEFiRet;
 
-	assert(pData != NULL);
-	assert(pData->f_hmysql == NULL);
+	ASSERT(pData != NULL);
+	ASSERT(pData->f_hmysql == NULL);
 
 	pData->f_hmysql = mysql_init(NULL);
 	if(pData->f_hmysql == NULL) {
-		logerror("can not initialize MySQL handle");
+		errmsg.LogError(NO_ERRCODE, "can not initialize MySQL handle");
 		iRet = RS_RET_SUSPENDED;
 	} else { /* we could get the handle, now on with work... */
 		/* Connect to database */
@@ -167,7 +157,7 @@ static rsRetVal initMySQL(instanceData *pData, int bSilent)
 		}
 	}
 
-	return iRet;
+	RETiRet;
 }
 
 
@@ -179,8 +169,13 @@ rsRetVal writeMySQL(uchar *psz, instanceData *pData)
 {
 	DEFiRet;
 
-	assert(psz != NULL);
-	assert(pData != NULL);
+	ASSERT(psz != NULL);
+	ASSERT(pData != NULL);
+
+	/* see if we are ready to proceed */
+	if(pData->f_hmysql == NULL) {
+		CHKiRet(initMySQL(pData, 0));
+	}
 
 	/* try insert */
 	if(mysql_query(pData->f_hmysql, (char*)psz)) {
@@ -200,7 +195,7 @@ finalize_it:
 		pData->uLastMySQLErrno = 0; /* reset error for error supression */
 	}
 
-	return iRet;
+	RETiRet;
 }
 
 
@@ -233,15 +228,13 @@ CODE_STD_STRING_REQUESTparseSelectorAct(1)
 	if(*p == '>') {
 		p++; /* eat '>' '*/
 	} else if(!strncmp((char*) p, ":ommysql:", sizeof(":ommysql:") - 1)) {
-		p += sizeof(":ommysql:"); /* eat indicator sequence */
+		p += sizeof(":ommysql:") - 1; /* eat indicator sequence  (-1 because of '\0'!) */
 	} else {
 		ABORT_FINALIZE(RS_RET_CONFLINE_UNPROCESSED);
 	}
 
 	/* ok, if we reach this point, we have something for us */
-	if((iRet = createInstance(&pData)) != RS_RET_OK)
-		goto finalize_it;
-
+	CHKiRet(createInstance(&pData));
 
 	/* rger 2004-10-28: added support for MySQL
 	 * >server,dbname,userid,password
@@ -277,16 +270,21 @@ CODE_STD_STRING_REQUESTparseSelectorAct(1)
 	 * Retries make no sense. 
 	 */
 	if (iMySQLPropErr) { 
-		logerror("Trouble with MySQL connection properties. -MySQL logging disabled");
+		errmsg.LogError(NO_ERRCODE, "Trouble with MySQL connection properties. -MySQL logging disabled");
 		ABORT_FINALIZE(RS_RET_INVALID_PARAMS);
 	} else {
-		CHKiRet(initMySQL(pData, 0));
+		pData->f_hmysql = NULL; /* initialize, but connect only on first message (important for queued mode!) */
 	}
 
 CODE_STD_FINALIZERparseSelectorAct
 ENDparseSelectorAct
 
 
+BEGINmodExit
+CODESTARTmodExit
+ENDmodExit
+
+
 BEGINqueryEtryPt
 CODESTARTqueryEtryPt
 CODEqueryEtryPt_STD_OMOD_QUERIES
@@ -295,8 +293,9 @@ ENDqueryEtryPt
 
 BEGINmodInit()
 CODESTARTmodInit
-	*ipIFVersProvided = 1; /* so far, we only support the initial definition */
+	*ipIFVersProvided = CURR_MOD_IF_VERSION; /* we only support the current interface specification */
 CODEmodInit_QueryRegCFSLineHdlr
+	CHKiRet(objUse(errmsg, CORE_COMPONENT));
 ENDmodInit
 /*
  * vi:set ai:
